# With JB mods for CUM consumption ...
# Original script by Trystan Lea ref: https://community.openenergymonitor.org/t/python-script-to-import-octopus-agile-consumption-data-into-emoncms/15230
# Requires Python 3
# To run the script in an SSH terminal do:  /usr/bin/python3 /home/pi/script-name.py (octopus_consumption_TL+JB_v2.py)
# To run the script as a daily cronjob do:  exactly the same
# Unlike other 'devices' which provide INPUT data as it occurs at n secs intervals, Octopus is providing a FEED of historical 30mins consumption data
# In consequence, it is necessary to FIRST create emoncms FEEDs (see Step 1 below) with a chosen tag, names & interval. The ID NUMBER for these FEEDs must then be entered in the script below at: feedid_A and feedid_B

import sys, requests, json 
from datetime import datetime

# Emoncms server configuration
emoncms_apikey = ""  # As appropriate
feedid_A = X   # As below, separately create FEEDs to receive the data.  Hover over the feed and enter the FEED ID NUMBER here
feedid_B = Y   # ditto
server = "http://127.0.0.1"    # Generic - adjust if necessary

# Agile API configuration
# Below are the inputs for the location
mpan = nnn   # Do not use quotes
serial_number = ""
agile_apikey = ""

# Agile request parameters
params = {'page':1,'order_by':'period','page_size':25000}

# Step 1: Create feeds via API call or use the input interface in emoncms to create manually
# http://IP-ADDRESS/feed/create.json?tag=Octopus Agile&name=consumption&datatype=1&engine=5&options={"interval":"1800"}&unit=Wh
# http://IP-ADDRESS/feed/create.json?tag=Octopus Agile&name=CUM consumption&datatype=1&engine=5&options={"interval":"1800"}&unit=Wh
# Per JB - This Step must be done before running the script - FEED (tag Octopus Agile and 'subset' names (consumption & CUM consumption) are created before the script is run - see Comments above

# Step 2: Fetch feed meta data to find last data point time and value and last CUM value
result = requests.get(server+"/feed/getmeta.json",params={'id':feedid_A,'apikey':emoncms_apikey})
meta = json.loads(result.text)
print("Feed meta data:\t\t"+result.text)

result = requests.get(server+"/feed/value.json",params={'id':feedid_B,'apikey':emoncms_apikey})
cum = json.loads(result.text)
print("Feed data CUM:\t\t"+str(cum))
if cum is None: cum = 0    # Applies the first time the script is run

if meta['npoints']>0:  
    end_time = meta['start_time'] + (meta['interval'] * meta['npoints'])
    params['period_from'] = datetime.fromtimestamp(end_time).astimezone().isoformat()
    print("Request from:\t\t"+params['period_from'])
	
# Step 3: Request history from Octopus
url = "https://api.octopus.energy/v1/electricity-meter-points/%s/meters/%s/consumption/" % (mpan,serial_number)
result = requests.get(url,params=params,auth=(agile_apikey,''))
data = json.loads(result.text)

if not data: sys.exit(0)
if not 'results' in data: sys.exit(0)

dp_received = len(data['results'])
print("Number of data points:\t%s" % dp_received);

# Step 4: Process history into data arrays for emoncms
data_out_A = []
data_out_B = []
 
for dp in data['results']:
    time = int(datetime.timestamp(datetime.strptime(dp['interval_start'],"%Y-%m-%dT%H:%M:%S%z")))
    value = dp['consumption']*1000
    cum = cum + value 
    print(str(time)+" "+str(value)+" "+str(cum))
    data_out_A.append([time,value])
    data_out_B.append([time,cum])

# Step 5: Send data to emoncms
if len(data_out_A):
    print("Posting data to emoncms")
    result = requests.post(server+"/feed/insert.json",params={'id':feedid_A,'apikey':emoncms_apikey},data={'data':json.dumps(data_out_A)})
    result = requests.post(server+"/feed/insert.json",params={'id':feedid_B,'apikey':emoncms_apikey},data={'data':json.dumps(data_out_B)})